@@ -13,7 +13,7 @@ at the top-level directory.
  * \brief Definitions which are precision-neutral
  *
  * <pre>
- * -- Distributed SuperLU routine (version 9.0) --
+ * -- Distributed SuperLU routine (version 9.1.0) --
  * Lawrence Berkeley National Lab, Univ. of California Berkeley.
  * November 1, 2007
  *
@@ -34,6 +34,7 @@ at the top-level directory.
  *     November 12, 2022   version 8.1.2
  *     November 17, 2023   version 8.2.1
  *     May 8, 2024         version 9.0.0
+ *     November 17, 2024   version 9.1.0
  * </pre>
  */
 
@@ -87,9 +88,9 @@ at the top-level directory.
  * Versions 4.x and earlier do not include a #define'd version numbers.
  */
 #define SUPERLU_DIST_MAJOR_VERSION     9
-#define SUPERLU_DIST_MINOR_VERSION     0
+#define SUPERLU_DIST_MINOR_VERSION     1
 #define SUPERLU_DIST_PATCH_VERSION     0
-#define SUPERLU_DIST_RELEASE_DATE      "May 8, 2024"
+#define SUPERLU_DIST_RELEASE_DATE      "November 10, 2024"
 
 #include "superlu_dist_config.h"
 
@@ -440,7 +441,7 @@ typedef struct {
  *
  * (xsup,supno) describes mapping between supernode and column:
  *	xsup[s] is the leading column of the s-th supernode.
- *      supno[i] is the supernode no to which column i belongs;
+ *      supno[j] is the supernode number to which column j belongs;
  *	e.g.   supno 0 1 2 2 3 3 3 4 4 4 4 4   (n=12)
  *	        xsup 0 1 2 4 7 12
  *	Note: dfs will be performed on supernode rep. relative to the new
@@ -737,6 +738,8 @@ typedef struct {
     yes_no_t      PivotGrowth;
     yes_no_t      ConditionNumber;
     rowperm_t     RowPerm;
+    yes_no_t      SolveOnly;      /* Treat the input matrix A as the L & U factors */
+    int           ILU_level;      /* level-based ILU */
     int 	  ILU_DropRule;
     double	  ILU_DropTol;    /* threshold for dropping */
     double	  ILU_FillFactor; /* gamma in the secondary dropping */
@@ -744,6 +747,7 @@ typedef struct {
     double	  ILU_FillTol;    /* threshold for zero pivot perturbation */
     milu_t	  ILU_MILU;
     double	  ILU_MILU_Dim;   /* Dimension of PDE (if available) */
+    yes_no_t      UserDefineSupernode; /* Allow users to define supernodes */
     yes_no_t      ParSymbFact;
     yes_no_t      ReplaceTinyPivot; /* used in SuperLU_DIST */
     yes_no_t      SolveInitialized;
@@ -1119,6 +1123,8 @@ extern int_t symbfact_SubInit(superlu_dist_options_t *options,
 extern int_t symbfact_SubXpand(int_t, int_t, int_t, MemType, int_t *,
 			       Glu_freeable_t *);
 extern int symbfact_SubFree(Glu_freeable_t *);
+extern int_t ilu_level_symbfact(superlu_dist_options_t *, SuperMatrix *, int_t *,
+			      int_t *, Glu_persist_t *, Glu_freeable_t *);
 extern void    countnz_dist (const int_t, int_t *, int_t *, int_t *,
 			     Glu_persist_t *, Glu_freeable_t *);
 extern int64_t fixupL_dist (const int_t, const int_t *, Glu_persist_t *,
@@ -1160,6 +1166,7 @@ extern void  quickSort( int_t*, int_t, int_t, int_t);
 extern void  quickSortM( int_t*, int_t, int_t, int_t, int_t, int_t);
 extern int_t partition( int_t*, int_t, int_t, int_t);
 extern int_t partitionM( int_t*, int_t, int_t, int_t, int_t, int_t);
+extern int count_swaps(int_t perm[], int n);
 
 extern int compareInt_t(void *a, void *b);
 extern int compareInt(void *a, void *b);
@@ -1271,14 +1278,14 @@ typedef struct
 
 #endif
 
-extern void C_RdTree_Create(C_Tree* tree, MPI_Comm comm, int* ranks, int rank_cnt, int msgSize, char precision);
+// extern void C_RdTree_Create(C_Tree* tree, MPI_Comm comm, int* ranks, int rank_cnt, int msgSize, char precision);
 extern void C_RdTree_Create_nv(C_Tree* tree, MPI_Comm comm, int* ranks, int rank_cnt, int msgSize, char precision, int* needrecvrd,int* needsendrd);
 extern void C_RdTree_Nullify(C_Tree* tree);
 extern yes_no_t C_RdTree_IsRoot(C_Tree* tree);
 extern void C_RdTree_forwardMessageSimple(C_Tree* Tree, void* localBuffer, int msgSize);
 extern void C_RdTree_waitSendRequest(C_Tree* Tree);
 
-extern void C_BcTree_Create(C_Tree* tree, MPI_Comm comm, int* ranks, int rank_cnt, int msgSize, char precision);
+// extern void C_BcTree_Create(C_Tree* tree, MPI_Comm comm, int* ranks, int rank_cnt, int msgSize, char precision);
 extern void C_BcTree_Create_nv(C_Tree* tree, MPI_Comm comm, int* ranks, int rank_cnt, int msgSize, char precision, int* needrecv);
 extern void C_BcTree_Nullify(C_Tree* tree);
 extern yes_no_t C_BcTree_IsRoot(C_Tree* tree);
@@ -1298,7 +1305,7 @@ extern void treeImbalance3D(gridinfo3d_t *grid3d, SCT_t* SCT);
 extern void slu_SCT_printComm3D(gridinfo3d_t *grid3d, SCT_t* SCT);
 extern int_t zAllocBcast(int_t size, void** ptr, gridinfo3d_t* grid3d);
 extern int_t zAllocBcast_gridID(int_t size, void** ptr, int_t gridID, gridinfo3d_t* grid3d);
-extern void permCol_SymbolicFact3d(superlu_dist_options_t *options, int_t n, SuperMatrix *GA, int_t *perm_c, int_t *etree, 
+extern void permCol_SymbolicFact3d(superlu_dist_options_t *options, int n, SuperMatrix *GA, int_t *perm_c, int_t *etree, 
                            Glu_persist_t *Glu_persist, Glu_freeable_t *Glu_freeable, SuperLUStat_t *stat,
 						   superlu_dist_mem_usage_t*symb_mem_usage,
 						   gridinfo3d_t* grid3d);
